﻿MapDistance = LibStub("AceAddon-3.0"):NewAddon("MapDistance", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

local maxFacingDiff = 0.01
local EPS = 0.002
local count = 5
local interval = 0.1

function MapDistance:OnInitialize()    
    self.db = LibStub("AceDB-3.0"):New("MapDistanceDB")
    if self.db.global.areas == nil then
        self.db.global.areas = {}
    end
    self.facing=0
    self.mapstack={}
    self.mapstack.i=0
end

function MapDistance:OnEnable()
    self:RegisterChatCommand("md", "chat")
    self:RegisterChatCommand("MapDistance", "chat")
    self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "NewArea")
    self:RegisterEvent("ZONE_CHANGED_INDOORS", "NewZone")
    self:RegisterEvent("ZONE_CHANGED", "NewZone")
    self:SetupArea()
    
end

function MapDistance:chat(input)
    local arg1, arg2 = self:GetArgs(input, 2)
    if arg1 == "-info" then 
        self:mapinfo()
    else
        self:dist(arg1, arg2)
    end
end

function MapDistance:MapBegin()
    --self:Print("!!!"..self.mapstack.i)
    self.mapstack.i=self.mapstack.i+1
    if self.mapstack.i == 1 then
        self.mapstack.zone = GetCurrentMapZone()
        self.mapstack.continent = GetCurrentMapContinent()
        self.mapstack.level = GetCurrentMapDungeonLevel()
        local info = GetMapInfo()
        SetMapToCurrentZone()
        if (self.mapstack.zone == GetCurrentMapZone()
                and self.mapstack.continent == GetCurrentMapContinent()
                and self.mapstack.level == GetCurrentMapDungeonLevel())
                or (self.mapstack.continent == -1 and info ~= nil)
                then
            self.mapstack.skip = true
        end
    end
end

function MapDistance:MapEnd()
    self.mapstack.i = self.mapstack.i-1
    if self.mapstack.i == 0 then
        if self.mapstack.skip then
            self.mapstack.skip = false
            return
        end
        SetMapZoom(self.mapstack.continent, self.mapstack.zone)
        SetDungeonMapLevel(self.mapstack.level)
    end    
end

function MapDistance:mapinfo(input)
    --self:MapBegin()
    self:Print("MapAreaID: "..GetCurrentMapAreaID())
    self:Print("Continent: "..GetCurrentMapContinent())
    self:Print("MapDungeonLevel: "..GetCurrentMapDungeonLevel().."/"..GetNumDungeonMapLevels())
    self:Print("MapZone: "..GetCurrentMapZone())
    if GetMapInfo() ~= nil then
        self:Print("MapInfo: "..GetMapInfo())
    end
    local posx, posy = GetPlayerMapPosition("player")
    self:Print("X: "..posx)
    self:Print("Y: "..posy)
    self:Print("Facing: "..GetPlayerFacing())
    self:Print("Facing2: "..self.facing)
    if self.db.global.areas[GetMapInfo()] then
        self:Print("Scale: "..self:GetScale())
        self:Print("eps: "..self.db.global.areas[GetMapInfo()][GetCurrentMapDungeonLevel()].eps)
    end
    
    --self:MapEnd()
end

function MapDistance:NewArea()
    
    --self:Print("new Area")
    self:SetupArea()
end

function MapDistance:GetScale()
    --self:MapBegin()
    local ret = 0
    if self.db.global.areas[GetMapInfo()] == nil or self.area ~= GetMapInfo() or self.level ~= GetCurrentMapDungeonLevel() then        
        self:SetupArea()
    end
    ret = self.db.global.areas[GetMapInfo()][GetCurrentMapDungeonLevel()].scale
    --self:MapEnd()
    return ret
end

function MapDistance:NewZone()
    --self:Print("new Zone")
    self:MapBegin()
    if self.level ~= GetCurrentMapDungeonLevel() then
        self:SetupArea()
    end
    self:MapEnd()
end

function MapDistance:SetupArea()
    self:MapBegin()
    self:CancelTimer(self.timer, true)
    self.area = GetMapInfo()
    self.level = GetCurrentMapDungeonLevel()
    self.count = 0
    self.scale = 0
    self.eps = 0
    if self.db.global.areas[GetMapInfo()] == nil then
        self.db.global.areas[GetMapInfo()] = {}
        for i=1, GetNumDungeonMapLevels() do
            self.db.global.areas[GetMapInfo()][i] = {scale = 0; eps = 100}
        end
        self.db.global.areas[GetMapInfo()][self.level] = {scale = 0; eps = 100}
    end
    if self.db.global.areas[self.area][self.level].eps > EPS then
        self.timer = self:ScheduleRepeatingTimer("CalculateScaleFromMovement", interval)
    end
    self:MapEnd()
end

function MapDistance:CalculateScaleFromMovement()
    self:MapBegin()
    local x,y = GetPlayerMapPosition("player")
    if self.area ~= GetMapInfo() or IsSwimming() or IsFlying() or ( x == 0 and y == 0 ) or x == nil or y == nil then
        self.lastPosition = nil
        self:MapEnd()
        return 
    end
    if self.lastPosition == nil or self.lastPosition.facing ~= GetPlayerFacing() or
            self.lastPosition.speed ~= GetUnitSpeed("player") or
            (self.lastPosition.x == x and self.lastPosition.y == y) then
        self:SavePosition()
        self:MapEnd()
        return
    end
    --self:Print("x1="..self.lastPosition.x.." y1="..self.lastPosition.y.." x2="..x.." y2="..y)
    self.facing = self:GetAngle(self.lastPosition.x, self.lastPosition.y, x, y)
    if not self:EqualsA(GetPlayerFacing(), self.facing,  maxFacingDiff) then
        self:SavePosition()
        self:MapEnd()
        return
    end
    
    local dtime = GetTime()-self.lastPosition.time
    local d = self:GetDistanceMap(self.lastPosition.x, self.lastPosition.y, x, y)
    local scale = dtime*GetUnitSpeed("player")/d
    
    if self.count == 0 then
        self.count = 1
        self.scale = scale
        self:MapEnd()
        return
    end
    
    local eps = self:GetEps(self.scale, scale)
    if eps > self.eps then self.eps = eps end
    self.scale = (self.scale * self.count + scale) / (self.count + 1)
    self.count = self.count+1
    
    self:SavePosition()
    
    local area = self.db.global.areas[GetMapInfo()][GetCurrentMapDungeonLevel()]
    
    if self.count == count then
        if self.eps < area.eps then
            area.scale = self.scale
            area.eps = self.eps
        end
        self.count = 0
        self.eps = 0
        self. scale = 0
        if area.eps < EPS then
            self:CancelTimer(self.timer, true)
        end
    end
    self:MapEnd()
end

function MapDistance:GetAngle(x1, y1, x2, y2)
    local dx = (y1 - y2)*2/3
    local dy = x1 - x2
    local a = math.atan2(dy,dx)
    if a < 0 then a = a + 2*math.pi end
    return a
end

function MapDistance:GetSqrDistanceMap(x1, y1, x2, y2)
    if (x1 == 0 and y1 == 0) or (x2 == 0 and y2 == 0) then
        return 2
    end
    local dx = x2 - x1
    local dy = (y1 - y2) * 2/3
    return dx^2 + dy^2
end

function MapDistance:GetDistanceMap(x1, y1, x2, y2)
    return math.sqrt(self:GetSqrDistanceMap(x1, y1, x2, y2))
end

function MapDistance:GetSqrDistanceYards(x1, y1, x2, y2)
    --self:Print(self:GetScale())
    return self:GetSqrDistanceMap(x1, y1, x2, y2) * self:GetScale()^2 
end

function MapDistance:GetDistanceYards(x1, y1, x2, y2)
    return self:GetDistanceMap(x1, y1, x2, y2) * self:GetScale()
end

function MapDistance:GetUnitSqrDistance(unit1, unit2)
    --self:MapBegin()
    local x1, y1 = GetPlayerMapPosition(unit1)
    local x2, y2 = GetPlayerMapPosition(unit2)
    --self:MapEnd()
    return self:GetSqrDistanceYards(x1, y1, x2, y2)
end

function MapDistance:GetUnitDistance(unit1, unit2)
    --self:MapBegin()
    local x1, y1 = GetPlayerMapPosition(unit1)
    local x2, y2 = GetPlayerMapPosition(unit2)
    --self:MapEnd()
    return GetDistanceYards(x1, y1, x2, y2)
end

function MapDistance:dist(unit1, unit2)
    if unit2 == nil then
        unit2 = "player"
    end
    if unit1 == nil then
        unit1 = "target"
    end
    --self:MapBegin()
    local px, py = GetPlayerMapPosition(unit1)
    local tx, ty = GetPlayerMapPosition(unit2)
    --self:MapEnd()
    if GetUnitName(unit1) then
        unit1 = GetUnitName(unit1)
    end
    if GetUnitName(unit2) then
        unit2 = GetUnitName(unit1)
    end
    
    if px == 0 and py == 0 then       
        self:Print("Can't get the position of "..unit1..".")
        return
    end
    if tx == 0 and ty == 0 then

        self:Print("Can't get the position of "..unit2..".")
        return
    end
    self:Print(self:GetDistanceYards(px, py, tx, ty).." yards")
end

function MapDistance:CalculateScaleFromRange()
    
end

function MapDistance:EqualsP(a, b, e)
    if b == 0 and a == 0 then return true end
    if b == 0 and a ~= 0 then return false end
    if math.abs(a/b - 1) < e then return true end
    return false
end

function MapDistance:EqualsA(a, b, e)
    if math.abs(a-b) < e then return true end
    return false
end

function MapDistance:GetEps(a, b)
    if b == 0 then return 100 end
    return math.abs(a/b - 1)
end

function MapDistance:SavePosition()
    self:MapBegin()
    self.lastPosition = {}
    self.lastPosition.x, self.lastPosition.y = GetPlayerMapPosition("player")
    self.lastPosition.facing = GetPlayerFacing()
    self.lastPosition.time = GetTime()
    self.lastPosition.speed = GetUnitSpeed("player")
    self.lastPosition.area = GetMapInfo()
    self.lastPosition.level = GetCurrentMapDungeonLevel()
    self:MapEnd()
end