﻿POH = LibStub("AceAddon-3.0"):NewAddon("POH", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

local md = _G.MapDistance

function POH:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("POHDB")
end

function POH:OnEnable()
    self.guid={} -- guid -> raidid / partyid
    self.raidid={} -- raidid -> group
    self.group={}
    self.value={} -- raidid -> value
    self.health={} -- raidid -> -health
    self.inrange={} -- raidid -> inrange

    if self.db.global.cit ~= nil then
        self.crit=self.db.global.crit
        self.glyph=self.db.global.glyph
    else
        self.crit=false
        self.glyph=false
    end
    
    self:SetupHealing()
    self:SetupRoster()
    
    self:RegisterEvent("PLAYER_DAMAGE_DONE_MODS", "SetupHealing")
    self:RegisterEvent("PLAYER_TALENT_UPDATE", "SetupHealing")
    self:RegisterEvent("RAID_ROSTER_UPDATE", "SetupRoster")
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "SetupRoster")
    self:RegisterEvent("ZONE_CHANGED_NEW_AREA","SetupRoster")
    
    self:RegisterChatCommand("poh", "chat")
end

function POH:chat(input)
    local arg = self:GetArgs(input, 1)
    if arg == "-crit" then
        self.db.global.crit = not self.db.global.crit
        if self.db.global.crit then
            self:Print("crit = true")
        else
            self:Print("crit = false")
        end
    elseif arg == "-glyph" then
        self.db.global.glyph = not self.db.global.glyph
        if self.db.global.glyph then
            self:Print("glyph = true")
        else
            self:Print("glyph = false")
        end
    else
        if arg == nil then arg = "target" end
        local name = GetUnitName(arg)
        if name == nil then name = arg end
        self:CalculateValues()
        
        local value = self:GetValue("target")
        if value then
            self:Print(name)
            self:Print("value: "..self:GetValue("target"))
            self:Print("inrange: "..self:GetInRange("target"))
        else
            self:Print(name.." is not in your raid or group!")
        end
    end
end

function POH:SetupRoster()
    local g
    wipe(self.guid)
    wipe(self.raidid)
    wipe(self.group)
    wipe(self.value)
    wipe(self.health)
    wipe(self.inrange)
    self.raid=UnitInRaid("player")
    
    if self.raid then
        for i=1, GetNumRaidMembers() do
            _,_, g,_,_,_,_,_,_,_,_= GetRaidRosterInfo(i)
            if self.group[g] == nil then
                self.group[g]={}
            end
            table.insert(self.group[g],i)
            self.guid[UnitGUID("raid"..i)]=i
            self.raidid[i]=g
        end
    else
        table.insert(self.group,"player")
        self.guid[UnitGUID("player")] = 1
        for i=1, GetNumPartyMembers() do
            table.insert(self.group,"party"..i)
            self.guid[UnitGUID("party"..i)]=i+1
        end
    end
end

function POH:SetupHealing()
    local rank1
    local rank2
    local rank3
    local class
    _ , class = UnitClass("player")
    if class ~= "PRIEST" then
        self.heal=4000
        self.range = 32.5 ^ 2
        return
    end
    _, _, _, _, rank1, _, _, _, _, _ = GetTalentInfo(2, 16, false, false, nil)
    _, _, _, _, rank2, _, _, _, _, _ = GetTalentInfo(2, 26, false, false, nil)
    _, _, _, _, rank3, _, _, _, _, _ = GetTalentInfo(2, 9, false, false, nil)
    self.heal = (2209+(0.526*GetSpellBonusHealing()))*(1+(rank1+rank2)*0.02)
    self.range = (30+3*rank3+2.5)^2
    if self.db.global.crit then
        self.heal = self.heal*1.5
    end
    
    if self.db.global.glyph then
        for i=1, 6 do
            local enabled, glyphType, glyphSpell, icon = GetGlyphSocketInfo(i)
            if glyphSpell == 55680 then
                self.heal = self.heal*1.2
                return
            end
        end
    end

end

function POH:GetGroup(unit)
    if UnitInRaid(unit) then
        return self.raidid[self.guid[UnitGUID(unit)]]
    end
end

function POH:CalculateValues()
    --wipe(self.value)
    --wipe(self.health)
    local g
    md:MapBegin()
    if self.raid then
        for i = 1, GetNumRaidMembers() do
            self.health[i] = self:UnitHealth("raid"..i)
            self.value[i] = self.health[i]
            self.inrange[i] = 1
        end
        for k=1, 8 do
            if self.group[k] ~= nil then
                local g = self.group[k]
                for i = 1, #g do               
                    for j = i+1 , #g do
                        if md:GetUnitSqrDistance("raid"..g[i],"raid"..g[j]) < self.range and not UnitIsDeadOrGhost("raid"..g[i]) and not UnitIsDeadOrGhost("raid"..g[j]) then
                            self.value[g[i]] = self.value[g[i]] + self.health[g[j]]
                            self.inrange[g[i]] = self.inrange[g[i]] + 1
                            self.value[g[j]] = self.value[g[j]] + self.health[g[i]]
                            self.inrange[g[j]] = self.inrange[g[j]] + 1
                        end
                    end
                end
            end
        
        end
    else
        g = self.group
        for i = 1, #g do
            self.health[i] = self:UnitHealth(g[i])
            self.value[i] = self.health[i]
            self.inrange[i] = 1
        end
        for i = 1, #g do               
            for j = i+1 , #g do
                if md:GetUnitSqrDistance(g[i],g[j]) < self.range and not UnitIsDeadOrGhost(g[i]) and not UnitIsDeadOrGhost(g[j]) then
                    self.value[i] = self.value[i] + self.health[j]
                    self.inrange[i] = self.inrange[i] + 1
                    self.value[j] = self.value[j] + self.health[i]
                    self.inrange[j] = self.inrange[j] + 1
                end
            end
        end
    end
    md:MapEnd()
end

function POH:GetValue(unit)
    if UnitInRaid(unit) or UnitInParty(unit) then
        return self.value[self.guid[UnitGUID(unit)]]
    else
        return nil
    end
end

function POH:GetInRange(unit)
    if UnitInRaid(unit) or UnitInParty(unit) then
        return self.inrange[self.guid[UnitGUID(unit)]]
    else
        return 0
    end
end

function POH:UnitHealth(unit)
    if UnitIsDeadOrGhost(unit) then return 0 end
    if heal == 0 then return 0.2 end
    local t = (UnitHealthMax(unit)-UnitHealth(unit))/self.heal
    if t>1 then
        t=1
    end
    if t<0 then
        t=0
    end
    return t*0.2
end

